/**************************************************************
	Gakken Otonano kagaku vol.24
	4bit microcomputer GMC-4 serial interface
	 for PIC16F648A/628(A)
 **************************************************************
	author air_variable
	http://www.ne.jp/asahi/air/variable/
	version 1.00
	Tools:
	MPLAB v8.33
	Hitech PIC C pro9.65pl1
---------------------------------------------------------------
Date	:Time	Version	Build	Note
20090818:0700	1.00	---		begin the program

---------------------------------------------------------------
	Pin assignment
	1:RA2:O:KEY_ROW2
	2:RA3:O:KEY_ROW3
	3:RA4:I:(not used)
	4:MCLR:I:PICKIT2-1(W/10kPU)
	5:VSS:GND
	6:RB0:O:KEY_nCOL0
	7:RB1:I:Receive serial
	8:RB2:O:Transmit serial
	9:RB3:O:KEY_nCOL1

	18:RA1:O:KEY_ROW1
	17:RA0:O:KEY_ROW0
	16:RA7:O:Remort_ON
	15:RA6:O:KEY_ROW4
	14:VDD:Power 3 to 5 Volt
	13:RB7:exclude use for pickit2
	12:RB6:exclude use for pickit2
	11:RB5:O:KEY_nCOL3
	10:RB4:O:KEY_nCOL2
	
	Key table
	  ROW 0 1 2 3 4
	nCOL0 C D E F A-SET(s)
	nCOL1 8 9 A B INCR(i or ',')
	nCOL2 4 5 6 7 RUN(r)
	nCOL3 0 1 2 3 RESET(x)

	extra command P or p pause 1sec	
	
	communication settings
	baud:300pps 8bit flow:none stop:1bit
	ӎ
	CɑM鎞140ȏɂȂƒʐMobt@̊֌WŎ肱ڂ܂B
	̏ꍇ́AɕăvO𑗐MĂB
	
**************************************************************/

#include <htc.h>
#include "sci.h"
//#include "delay.h"
#include "lcd.h"

#define MHz 000000
#define _XTAL_FREQ 4MHz



//Key control settings
#define	Set_key_column_0	TRISB =	TRISB;\
							PORTB &=  0b11111110;	//RB0=0
#define	Set_key_column_1	TRISB =	TRISB;\
							PORTB &=  0b11110111;	//RB3=0
#define	Set_key_column_2	TRISB =	TRISB;\
							PORTB &=  0b11101111;	//RB4=0
#define	Set_key_column_3	TRISB =	TRISB;\
							PORTB &=  0b11011111;	//RB5=0

#define Set_key_row_0		TRISA =	TRISA;\
							PORTA |=  0b00000001;	//RA0=1
#define Set_key_row_1		TRISA =	TRISA;\
							PORTA |=  0b00000010;	//RA1=1
#define Set_key_row_2		TRISA =	TRISA;\
							PORTA |=  0b00000100;	//RA2=1
#define Set_key_row_3		TRISA =	TRISA;\
							PORTA |=  0b00001000;	//RA3=1
#define Set_key_row_4		TRISA =	TRISA;\
							PORTA |=  0b01000000;	//RA6=1

#define	Release_key			TRISA =	TRISA;\
							PORTA &=  0b10110000;\
							TRISB =	TRISB;\
							PORTB |=  0b00111001;
#define Remort_ON			RA7 = 1;
#define Remort_OFF			RA7 = 0;


//Key timing settings
#define	Key_release_time	40	//ms
#define	Key_input_time		40	//ms
#define	Pause_time			50	//x10ms

//receive setting
#define USART_SPEED	300	//speed set
#define MAX_BUFF	80		//RX ring buffer size 2^n


//Grobal settings

extern void delay_10ms(unsigned int temp);

//Receive FIFO parameter
unsigned char Buffer_FIFO[MAX_BUFF];
unsigned char FIFO_write_pointer=0;
unsigned char FIFO_read_pointer=0;
unsigned char FIFO_datasize=0;


//__CONFIG(0x3F02);
__CONFIG(
	UNPROTECT & LVPDIS & BORDIS & MCLRDIS & PWRTEN & WDTDIS & INTIO
	);
//Global settings

void interrupt handler( void ){		//interrupt handle
	unsigned char temp;
	unsigned char data;
	//Handle RS232 Receive interrupt
	if( RCIF == 1 ){
		//receive data
		data = sci_GetByte();
		//set to FIFO
		RCIF = 0; //clear RS232 receive interrupt bit
		if(FIFO_datasize<MAX_BUFF){
			Buffer_FIFO[FIFO_write_pointer]=data;
			FIFO_write_pointer++;	//=(FIFO_write_pointer+1) & (MAX_BUFF-1);//ring buff 
			if ( FIFO_write_pointer == MAX_BUFF ) FIFO_write_pointer=0;
			FIFO_datasize++;
		}
	}
}



void delay_10ms(unsigned int temp){	//10ms multiple x times
	while(temp--){
		__delay_ms(10);
	}	
}
	

void Key_entry(unsigned char temp){
	Remort_ON;
//	__delay_ms(10);
	switch ( temp ){
		case 'p':
		case 'P':
			delay_10ms(Pause_time);
			return;
			//Set_key_row
		case '0':
		case '4':
		case '8':
		case 'c':
		case 'C':
			Set_key_row_0;
			break;
		case '1':
		case '5':
		case '9':
		case 'd':
		case 'D':
			Set_key_row_1;
			break;
		case '2':
		case '6':
		case 'a':
		case 'A':
		case 'e':
		case 'E':
			Set_key_row_2;
			break;
		case '3':
		case '7':
		case 'b':
		case 'B':
		case 'f':
		case 'F':
			Set_key_row_3;
			break;
		case 's':
		case 'i':
		case 'r':
		case 'x':
		case 'S':
		case 'I':
		case 'R':
		case 'X':
		case ',':	// delimit
			Set_key_row_4;
			break;
		default:
			break;
	}		

	switch ( temp ){	//Set_key_column
		case '0':
		case '1':
		case '2':
		case '3':
		case 'x':
		case 'X':
			Set_key_column_3;
			break;
		case '4':
		case '5':
		case '6':
		case '7':
		case 'r':
		case 'R':
			Set_key_column_2;
			break;
		case '8':
		case '9':
		case 'a':
		case 'A':
		case 'b':
		case 'B':
		case 'i':
		case 'I':
		case ',':	// delimit
			Set_key_column_1;
			break;
		case 'c':
		case 'd':
		case 'e':
		case 'f':
		case 's':
		case 'C':
		case 'D':
		case 'E':
		case 'F':
		case 'S':
			Set_key_column_0;
			break;
		default:
			Release_key	
			Remort_OFF;
			return;
	}

	__delay_ms(Key_input_time);
	Release_key;
	__delay_ms(Key_release_time);
	Remort_OFF;	// The internal key able to use it.

	return;
}		
		

//------------------main
main(){
	unsigned char temp;

//	Port initalize
//		  	  76543210
	PORTA = 0b00000000;	//Port Initalize
	PORTB = 0b00111001;	//Port Initalize
	TRISA = 0b00110000; //
	TRISB = 0b00000110; //
	CMCON = 0x07;		//PORTA is all digital I/O

	delay_10ms(10);	//1sec delay

//ʐM
	sci_Init( USART_SPEED,SCI_EIGHT);// Baud set and Bit set
	CREN =0;	//communication stop
	while (RCIF ==1){
		temp = sci_GetByte();		//Dummy read
	}
	//sci check
	if (sci_CheckOERR()) { //sci error
		while (!TRMT);
		CREN=1;
	}

	RCIE = 1;		//receive interrupt on
	PEIE = 1;		//Peripheral interrupt enable
	GIE  = 1;		//Global interrupt enable

	CREN =1;	//communication stop
	
	while(1){		//Loop forever 
		if (FIFO_datasize !=0){	// FIFO𒲍āAf[^݂ǂ݂ɍs
			temp = Buffer_FIFO[FIFO_read_pointer];
			FIFO_read_pointer++;	//=(FIFO_read_pointer+1)&(MAX_BUFF-1);
			if ( FIFO_read_pointer == MAX_BUFF ) FIFO_read_pointer=0;
			//ring buff
			FIFO_datasize--;
			Key_entry( temp );
			sci_PutByte(temp);

		}
	}
}
